@@ -20,7 +20,7 @@ from glob import glob
 from kook.utils import read_file, write_file
 
 package   = prop('package', 'Oktest')
-release   = prop('release', '0.11.0')
+release   = prop('release', '0.11.1')
 copyright = prop('copyright', "copyright(c) 2010-2011 kuwata-lab.com all rights reserved")
 license   = "MIT License"
 python    = prop('python', sys.executable)
@@ -132,7 +132,7 @@ def _run_oktest_test(c, ver, bin):
     #    f()
     #else:
     #    system(cmd)
-    system(cmd)
+    system_f(cmd)
 
 #def _invoke_test(c, ver, bin, fpath):
 #    def replacer_for_py24(s):
