@@ -3,22 +3,30 @@
 ###
 ### oktest.py -- new style test utility
 ###
-### $Release: 0.11.0 $
+### $Release: 0.11.1 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
 
 __all__ = ('ok', 'NOT', 'NG', 'not_ok', 'run', 'spec', 'test', 'fail', 'skip', 'todo', 'subject', 'situation', 'main')
-__version__ = "$Release: 0.11.0 $".split()[1]
+__version__ = "$Release: 0.11.1 $".split()[1]
 
 import sys, os, re, types, traceback, time, linecache
 
+ENCODING = 'utf-8'
+TERMINAL_WIDTH = 80
+
+
 python2 = sys.version_info[0] == 2
 python3 = sys.version_info[0] == 3
 if python2:
+    _unicode = unicode
+    _bytes   = str
     from cStringIO import StringIO
 if python3:
     xrange = range
+    _unicode = str
+    _bytes   = bytes
     from io import StringIO
 
 
@@ -901,6 +909,20 @@ class BaseReporter(Reporter):
         if out is not None and self._color is None:
             self._set_color(None)
 
+    def write(self, string):
+        try:
+            self.out.write(string)
+        except UnicodeDecodeError:
+            if isinstance(string, _bytes):
+                self.out.write(string.decode('utf-8'))
+            else:
+                raise
+        except UnicodeEncodeError:
+            if isinstance(string, _unicode):
+                self.out.write(string.encode('utf-8'))
+            else:
+                raise
+
     out = property(__get_out, __set_out)
 
     def clear_counts(self):
@@ -944,7 +966,7 @@ class BaseReporter(Reporter):
         min = int(int(dt) / 60)     # int / int is float on Python3
         sec = dt - (min * 60)
         elapsed = min and "%s:%06.3f" % (min, sec) or "%.3f" % sec
-        self.out.write("## %s  (%s sec)\n" % (self.counts2str(), elapsed))
+        self.write("## %s  (%s sec)\n" % (self.counts2str(), elapsed))
         self.out.flush()
 
     def enter_testclass(self, testclass):
@@ -1019,8 +1041,8 @@ class BaseReporter(Reporter):
             title = " > ".join(items)
         indicator = self.indicator(status)
         self.write_separator()
-        self.out.write("[%s] %s\n" % (indicator, title))
-        if desc: self.out.write(desc + "\n")
+        self.write("[%s] %s\n" % (indicator, title))
+        if desc: self.write(desc + "\n")
 
     def _get_testcase_header_items(self, testcase, testname):
         parent = self.get_testclass_name(testcase.__class__)
@@ -1042,17 +1064,20 @@ class BaseReporter(Reporter):
         filter = not config.debug and self._filter or None
         arr = format_traceback(ex, ex_traceback, filter=filter)
         for x in arr:
-            self.out.write(x)
-        errmsg = "%s: %s" % (ex_class.__name__, ex)
+            self.write(x)
+        try:
+            errmsg = "%s: %s" % (ex_class.__name__, ex)
+        except UnicodeEncodeError:
+            errmsg = unicode("%s: %s") % (ex_class.__name__, ex)
         tupl = errmsg.split("\n", 1)
         if len(tupl) == 1:
             first_line, rest = tupl[0], None
         else:
             first_line, rest = tupl
-        self.out.write(self.colorize(first_line, status) + "\n")
+        self.write(self.colorize(first_line, status) + "\n")
         if rest:
-            self.out.write(rest)
-            if not rest.endswith("\n"): self.out.write("\n")
+            self.write(rest)
+            if not rest.endswith("\n"): self.write("\n")
         self.out.flush()
 
     def report_exception_footer(self, testcase, testname, status, exc_info, context):
@@ -1061,16 +1086,23 @@ class BaseReporter(Reporter):
     def _print_temporary_str(self, string):
         if is_tty(self.out):
             #self.__string = string
-            self.out.write(string)
+            if not util._is_unicode(string):
+                string = string.decode(ENCODING)
+            if util.zenkaku_width(string) >= TERMINAL_WIDTH:
+                shorten = util.zenkaku_shorten(string, TERMINAL_WIDTH - 4)
+                #self.write(string)
+                self.write(shorten + '...')
+            else:
+                self.write(string)
             self.out.flush()
 
-    def _erase_temporary_str(self, _eraser="\b"*255):
+    def _erase_temporary_str(self):
         if is_tty(self.out):
             #n = len(self.__string) + 1    # why '+1' ?
-            #self.out.write("\b" * n)      # not work with wide-chars
+            #self.write("\b" * n)          # not work with wide-chars
             #self.out.flush()
             #del self.__string
-            self.out.write(_eraser)
+            self.write("\r")     # or "\r\033[K"
             self.out.flush()
 
     def report_spec_esception(self, testcase, testname, status, spec, context):
@@ -1080,8 +1112,8 @@ class BaseReporter(Reporter):
         parent, child, desc = self._get_testcase_header_items(testcase, testname)
         indicator = self.indicator(status)
         self.write_separator()
-        self.out.write("[%s] %s > %s > %s\n" % (indicator, parent, child, spec.desc))
-        if desc: self.out.write(desc + "\n")
+        self.write("[%s] %s > %s > %s\n" % (indicator, parent, child, spec.desc))
+        if desc: self.write(desc + "\n")
         #
         stacktrace = self._filter_stacktrace(spec._stacktrace, spec._traceback)
         self._print_stacktrace(stacktrace)
@@ -1103,10 +1135,12 @@ class BaseReporter(Reporter):
 
     def _print_stacktrace(self, stacktrace):
         for file, line, func, text in stacktrace:
-            self.out.write('  File "%s", line %s, in %s\n' % (file, line, func))
-            self.out.write('    %s\n' % text)
+            self.write('  File "%s", line %s, in %s\n' % (file, line, func))
+            self.write('    %s\n' % text)
 
     def colorize(self, string, kind):
+        if self._color is None:
+            self._set_color(None)
         if not self._color:
             return string
         if kind == ST_PASSED:  return util.Color.green(string, bold=True)
@@ -1121,7 +1155,7 @@ class BaseReporter(Reporter):
         return util.Color.yellow(string)
 
     def write_separator(self):
-        self.out.write(self.colorize(self.separator, "sep") + "\n")
+        self.write(self.colorize(self.separator, "sep") + "\n")
 
     def status_char(self, status):
         if not hasattr(self, '_status_chars'):
@@ -1201,7 +1235,7 @@ class VerboseReporter(BaseReporter):
 
     def enter_testclass(self, testclass):
         self._super.enter_testclass(self, testclass)
-        self.out.write("* %s\n" % self.colorize(self.get_testclass_name(testclass), "topic"))
+        self.write("* %s\n" % self.colorize(self.get_testclass_name(testclass), "topic"))
         self.out.flush()
 
     def enter_testcase(self, testcase, testname):
@@ -1220,7 +1254,7 @@ class VerboseReporter(BaseReporter):
         self._erase_temporary_str()
         indicator = self.indicator(status)
         desc = self.get_testcase_desc(testcase, testname)
-        self.out.write("  " * self.depth + "- [%s] %s%s\n" % (indicator, desc, s))
+        self.write("  " * self.depth + "- [%s] %s%s\n" % (indicator, desc, s))
         self.out.flush()
 
     def enter_testcontext(self, context):
@@ -1228,7 +1262,7 @@ class VerboseReporter(BaseReporter):
         s = context.desc
         if not (s.startswith("when ") or s == "else:"):
             s = self.colorize(s, "context")
-        self.out.write("  " * self.depth + "+ %s\n" % s)
+        self.write("  " * self.depth + "+ %s\n" % s)
         self.depth += 1
 
     def exit_testcontext(self, context):
@@ -1247,16 +1281,16 @@ class SimpleReporter(BaseReporter):
 
     def enter_testclass(self, testclass):
         self._super.enter_testclass(self, testclass)
-        self.out.write("* %s: " % self.colorize(self.get_testclass_name(testclass), "topic"))
+        self.write("* %s: " % self.colorize(self.get_testclass_name(testclass), "topic"))
         self.out.flush()
 
     def exit_testclass(self, *args):
-        self.out.write("\n")
+        self.write("\n")
         self._super.exit_testclass(self, *args)
 
     def exit_testcase(self, testcase, testname, status, exc_info):
         self._super.exit_testcase(self, testcase, testname, status, exc_info)
-        self.out.write(self.status_char(status))
+        self.write(self.status_char(status))
         self.out.flush()
 
 BaseReporter.register_class("simple", SimpleReporter)
@@ -1271,16 +1305,16 @@ class PlainReporter(BaseReporter):
 
     def exit_testclass(self, testclass, method_name, exc_info):
         if self._exceptions or exc_info:
-            self.out.write("\n")
+            self.write("\n")
         self._super.exit_testclass(self, testclass, method_name, exc_info)
 
     def exit_testcase(self, testcase, testname, status, exc_info):
         self._super.exit_testcase(self, testcase, testname, status, exc_info)
-        self.out.write(self.status_char(status))
+        self.write(self.status_char(status))
         self.out.flush()
 
     def exit_all(self):
-        self.out.write("\n")
+        self.write("\n")
         self._super.exit_all(self)
 
 BaseReporter.register_class("plain", PlainReporter)
@@ -1308,11 +1342,11 @@ class UnittestStyleReporter(BaseReporter):
 
     def exit_testcase(self, testcase, testname, status, exc_info):
         self._super.exit_testcase(self, testcase, testname, status, exc_info)
-        self.out.write(self.status_char(status))
+        self.write(self.status_char(status))
         self.out.flush()
 
     def exit_all(self):
-        self.out.write("\n")
+        self.write("\n")
         for tupl in self._exceptions:
             self.report_exceptions(*tupl)
         self._super.exit_all(self)
@@ -1326,9 +1360,9 @@ class UnittestStyleReporter(BaseReporter):
             parent = testcase.__class__.__name__
             child  = testname
         indicator = self.INDICATOR.get(status) or '???'
-        self.out.write("=" * 70 + "\n")
-        self.out.write("%s: %s#%s()\n" % (indicator, parent, child))
-        self.out.write("-" * 70 + "\n")
+        self.write("=" * 70 + "\n")
+        self.write("%s: %s#%s()\n" % (indicator, parent, child))
+        self.write("-" * 70 + "\n")
 
 BaseReporter.register_class("unittest", SimpleReporter)
 
@@ -1350,36 +1384,36 @@ class OldStyleReporter(BaseReporter):
         pass
 
     def enter_testcase(self, testcase, testname):
-        self.out.write("* %s.%s ... " % (testcase.__class__.__name__, testname))
+        self.write("* %s.%s ... " % (testcase.__class__.__name__, testname))
 
     def exit_testcase(self, testcase, testname, status, exc_info):
         if status == ST_PASSED:
-            self.out.write("[ok]\n")
+            self.write("[ok]\n")
         elif status == ST_FAILED:
             ex_class, ex, ex_traceback = exc_info
             flag = hasattr(ex, '_raised_by_oktest')
-            self.out.write("[NG] %s\n" % (flag and ex.errmsg or util.ex2msg(ex)))
+            self.write("[NG] %s\n" % (flag and ex.errmsg or util.ex2msg(ex)))
             def formatter(filepath, lineno, funcname, linestr):
                 return "   %s:%s: %s\n" % (filepath, lineno, linestr.strip())
             arr = format_traceback(ex, ex_traceback, filter=self._filter, formatter=formatter)
             for x in arr:
-                self.out.write(x)
+                self.write(x)
             if flag and getattr(ex, 'diff', None):
-                self.out.write(ex.diff)
+                self.write(ex.diff)
         elif status == ST_ERROR:
             ex_class, ex, ex_traceback = exc_info
-            self.out.write("[ERROR] %s: %s\n" % (ex_class.__name__, util.ex2msg(ex)))
+            self.write("[ERROR] %s: %s\n" % (ex_class.__name__, util.ex2msg(ex)))
             def formatter(filepath, lineno, funcname, linestr):
                 return "  - %s:%s:  %s\n" % (filepath, lineno, linestr.strip())
             arr = format_traceback(ex, ex_traceback, filter=self._filter, formatter=formatter)
             for x in arr:
-                self.out.write(x)
+                self.write(x)
         elif status == ST_SKIPPED:
-            self.out.write("[skipped]\n")
+            self.write("[skipped]\n")
         elif status == ST_TODO:
-            self.out.write("[TODO]\n")
+            self.write("[TODO]\n")
         #elif status == ST_UNEXPECTED:
-        #    self.out.write("[Unexpected]\n")
+        #    self.write("[Unexpected]\n")
         else:
             assert False, "UNREACHABLE: status=%r" % (status,)
 
@@ -1404,8 +1438,11 @@ def _dummy():
     __all__ = ('chdir', 'rm_rf')
 
     if python2:
+        _unicode = unicode
         def _is_string(val):
             return isinstance(val, (str, unicode))
+        def _is_unicode(val):
+            return isinstance(val, unicode)
         def _is_class(obj):
             return isinstance(obj, (types.TypeType, types.ClassType))
         def _is_unbound(method):
@@ -1416,8 +1453,11 @@ def _dummy():
             func = getattr(func, 'im_func', func)
             return func.func_code.co_firstlineno
     if python3:
+        _unicode = str
         def _is_string(val):
             return isinstance(val, (str, bytes))
+        def _is_unicode(val):
+            return isinstance(val, str)
         def _is_class(obj):
             return isinstance(obj, (type, ))
         def _is_unbound(method):
@@ -1561,6 +1601,29 @@ def _dummy():
             assert isinstance(u, str)
             return u
 
+    import unicodedata as _unicodedata
+    _CHAR_WIDTH = {'W': 2, 'F': 2, 'A': 2}
+
+    def zenkaku_width(unicode_string,
+                      _unicode=_unicode, _eaw=_unicodedata.east_asian_width, _widths=_CHAR_WIDTH):
+        if not isinstance(unicode_string, _unicode):
+            raise TypeError('unicode expected but got %s.' % type(unicode_string))
+        return sum( _widths.get(_eaw(ch), 1) for ch in unicode_string )
+
+    def zenkaku_shorten(unicode_string, max_width,
+                      _unicode=_unicode, _eaw=_unicodedata.east_asian_width, _widths=_CHAR_WIDTH):
+        if not isinstance(unicode_string, _unicode):
+            raise TypeError('unicode expected but got %s.' % type(unicode_string))
+        w = i = 0
+        for ch in unicode_string:
+            w += _widths.get(_eaw(ch), 1)   # 2 when zenkaku, 1 when hankaku
+            if w > max_width:
+                break
+            i += 1
+        else:
+            return unicode_string
+        return unicode_string[:i]
+
     from types import MethodType as _MethodType
 
     if python2:
