@@ -1,5 +1,5 @@
 ###
-### $Release: 0.11.0 $
+### $Release: 0.11.1 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -81,7 +81,7 @@ class Spec_TC(unittest.TestCase):
         assert isinstance(obj._stacktrace, list)
 
     def test___iter___1(sefl):
-        """emurates with-stmt when used with for-stmt."""
+        """emulates with-stmt when used with for-stmt."""
         called = []
         def enter(*args):
             called.append(('enter', args))
@@ -101,7 +101,7 @@ class Spec_TC(unittest.TestCase):
         ok (called[2]) == ('exit', (None, None, None))
 
     def test___call___1(self):
-        """emurates with-stmt when called as decorator."""
+        """emulates with-stmt when called as decorator."""
         arr = []
         def fn():
             obj = Spec('foo')
@@ -114,7 +114,7 @@ class Spec_TC(unittest.TestCase):
         ok (arr) == ["before", "after"]
 
     def test___call___2(self):
-        """emurates with-stmt when called as decorator."""
+        """emulates with-stmt when called as decorator."""
         arr = []
         def fn():
             obj = Spec('foo')
@@ -125,6 +125,9 @@ class Spec_TC(unittest.TestCase):
                 arr.append("after")
         ok (fn).raises(AssertionError, "2 == 1 : failed.")
         ok (arr) == ["before"]
+        #
+        if hasattr(self, '_oktest_specs'):
+            self._oktest_specs[:] = []
 
     def test___bool___1(self):
         """returns True when $SPEC is not set."""
