@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 ###
-### $Release: 0.11.0 $
+### $Release: 0.11.1 $
 ### $Copyright: copyright(c) 2010-2011 kuwata-lab.com all rights reserved $
 ### $License: MIT License $
 ###
@@ -11,7 +11,7 @@ import unittest
 import oktest
 from oktest import ok, NG, run
 from oktest import python2, python3
-from oktest.util import chdir, flatten, rm_rf
+from oktest.util import chdir, flatten, rm_rf, zenkaku_width, zenkaku_shorten
 
 
 available_with_statement = sys.version_info[0:2] > (2, 4)
@@ -295,6 +295,30 @@ class util_TC(unittest.TestCase):
             self.assertEqual(UnicodeDecodeError, ex.__class__)
             self.assertTrue("'utf8' codec can't decode byte 0x82 in position 0: invalid start byte", str(ex))
 
+    def test_zenkaku_width_1(self):
+        if python2:
+            _unistr = lambda s: s.decode('utf-8')
+        elif python3:
+            _unistr = lambda s: s
+        ok (zenkaku_width(_unistr("SOS"))) == 3
+        ok (zenkaku_width(_unistr("ハルヒ"))) == 6
+        ok (zenkaku_width(_unistr("SOS/ハルヒ"))) == 10
+
+    def test_zenkaku_shorten_1(self):
+        if python2:
+            _unistr = lambda s: s.decode('utf-8')
+        elif python3:
+            _unistr = lambda s: s
+        unicode_string = _unistr("SOS/ハルヒ")
+        ok (zenkaku_shorten(unicode_string, 4)) == _unistr("SOS/")
+        ok (zenkaku_shorten(unicode_string, 5)) == _unistr("SOS/")
+        ok (zenkaku_shorten(unicode_string, 6)) == _unistr("SOS/ハ")
+        ok (zenkaku_shorten(unicode_string, 7)) == _unistr("SOS/ハ")
+        ok (zenkaku_shorten(unicode_string, 8)) == _unistr("SOS/ハル")
+        ok (zenkaku_shorten(unicode_string, 9)) == _unistr("SOS/ハル")
+        ok (zenkaku_shorten(unicode_string, 10)) == _unistr("SOS/ハルヒ")
+        ok (zenkaku_shorten(unicode_string, 11)) == _unistr("SOS/ハルヒ")
+        ok (zenkaku_shorten(unicode_string, 12)) == _unistr("SOS/ハルヒ")
 
     def test_helper_1(self):
         """'help' is an alias of 'util'"""
